#include <Adafruit_NeoPixel.h>
#include <DFRobot_MCP23017.h>

#define NUMPIXELS   64        //灯数
#define LEDPIN      9         //LED引脚
#define DELAYVAL    50        //点亮速度
#define BRIGHTNESS  50        //灯亮度
#define OFF         0         //关灯
#define BLUE        0x0000ff  //蓝色
#define ORANGE      0xff9900  //橙色
#define RED         0xff0000  //红色

uint8_t x, y ,x0, y0;                    //坐标
int num[8];                          //新行号1-8数据
int num0[8];                         //旧行号1-8数据
const uint8_t conversion[8][2] = {1, 1, 2, 2, 4, 3, 8, 4, 16, 5, 32, 6, 64, 7, 128, 8};  //列号转换
uint8_t attack = 0;                      //2代表走子，3代表吃子

// 创建对象
Adafruit_NeoPixel LED(NUMPIXELS,  LEDPIN,  NEO_GRB + NEO_KHZ800);  //灯条

DFRobot_MCP23017 mcp1(Wire,  0x24);       //IO扩展板1
DFRobot_MCP23017 mcp2(Wire,  0x25);       //IO扩展板2
DFRobot_MCP23017 mcp3(Wire,  0x26);       //IO扩展板3
DFRobot_MCP23017 mcp4(Wire,  0x27);       //IO扩展板4

void LEDInit(){   //初始化灯
  LED.begin();
  LED.setBrightness(BRIGHTNESS);
}

void IOInit(){   //初始化扩展板
  uint8_t i;
  while(mcp1.begin() != 0);     //等待扩展板1初始化完成
  while(mcp2.begin() != 0);     //等待扩展板2初始化完成
  while(mcp3.begin() != 0);     //等待扩展板3初始化完成
  while(mcp4.begin() != 0);     //等待扩展板4初始化完成
  for(i = 0; i < 16; i++){
    mcp1.pinMode(i,  INPUT);     //初始化扩展板1所有IO
    mcp2.pinMode(i,  INPUT);     //初始化扩展板2所有IO
    mcp3.pinMode(i,  INPUT);     //初始化扩展板3所有IO
    mcp4.pinMode(i,  INPUT);     //初始化扩展板4所有IO
  }
}

void PiecesInit(){   //等待棋子摆放完成, 如果没有摆放完成将在里面循环
  uint8_t i,j;
  for(i = 1; i <= 8; i++){       //点亮摆放位置灯
    LEDSwitch(i, 1, RED, 0);
    LEDSwitch(i, 2, RED, 0);
  }
  for(i = 1; i <= 8; i++){
    LEDSwitch(i, 7, BLUE, 0);
    LEDSwitch(i, 8, BLUE, 0);
  }
  //等待棋子摆放完成
  while((mcp1.digitalRead(mcp1.eGPA) ||   //第1行
        mcp1.digitalRead(mcp1.eGPB)) ||  //第2行
        (mcp4.digitalRead(mcp4.eGPA) ||  //第7行
        mcp4.digitalRead(mcp4.eGPB)));   //第8行
  for(i = 1; i <= 8; i++){       //关闭摆放位置灯
    LEDSwitch(i, 1, OFF, 0);
    LEDSwitch(i, 2, OFF, 0);
  }
  for(i = 1; i <= 8; i++){
    LEDSwitch(i, 7, OFF, 0);
    LEDSwitch(i, 8, OFF, 0);
  }
  //开局灯光，正方形往外绽放
  i = 4, j = 5;
  for(;i >= 1; i--, j++){
    StartLED(i,j);
  }
}

void StartLED(uint8_t m, uint8_t n){    //开局灯光，正方形往外绽放
  uint8_t i, j;
  for(i = m; i <= n; i++){
    LEDSwitch(m, i, ORANGE, 0);
  }
  for(j = n; j >= m; j--){
    LEDSwitch(n, j, ORANGE, 0);
  }
  for(i = m; i <= n; i++){
    LEDSwitch(i, m, ORANGE, 0);
  }
  for(j = n; j >= m; j--){
    LEDSwitch(j, n, ORANGE, 0);
  }
  delay(100);
  for(i = m; i <= n; i++){
    LEDSwitch(m, i, OFF, 0);
  }
  for(j = n; j >= m; j--){
    LEDSwitch(n, j, OFF, 0);
  }
  for(i = m; i <= n; i++){
    LEDSwitch(i, m, OFF, 0);
  }
  for(j = n; j >= m; j--){
    LEDSwitch(j, n, OFF, 0);
  }
}

void CoordinatesInit(){   //初始化坐标
  uint8_t i = 0;
  for(; i < 8; i++){
    if(i > 2 && i < 6){
      num0[i] = num[i] = 255;
    }
    else{
      num0[i] = num[i] = 0;
    }
  }
}

void IOTraverse(){   //扫描所有IO口
  //存原来行数据
  uint8_t i = 0;
  for(; i < 8; i++){
    num0[i] = num[i];
  }
  //读取新行数据
  num[8] = {mcp1.digitalRead(mcp1.eGPA),   //第1行
                    mcp1.digitalRead(mcp1.eGPB),   //第2行
                    mcp2.digitalRead(mcp2.eGPA),   //第3行
                    mcp2.digitalRead(mcp2.eGPB),   //第4行
                    mcp3.digitalRead(mcp3.eGPA),   //第5行
                    mcp3.digitalRead(mcp3.eGPB),   //第6行
                    mcp4.digitalRead(mcp4.eGPA),   //第7行
                    mcp4.digitalRead(mcp4.eGPB)};   //第8行
}

uint8_t Coordinatecalculation(){   //位置、拿放计算
  int16_t UD;
  uint8_t i = 0,j = 0;
  for(;i < 8; i++){
    if(num0[i] ^ num[i]){
      UD  = num0[i] - num[i];  //UD(-):拿, 电平0变为1，对x0, y0赋值；UD(+):放, 电平1变为0，对x, y赋值
      if(UD > 0){
        for(;j < 8; j++){
          if(UD == conversion[j][0]){
            x = conversion[j][1];
            break;
          }
        }
        y = i + 1;
        return 1;      //棋子放下，返回1，点亮行走路径灯
      }else{
        UD *= -1;
        for(;j < 8; j++){
          if(UD == conversion[j][0]){
            x0 = conversion[j][1];
            break;
          }
        }
        y0 = i + 1;
        attack++;
        return 0;      //棋子拿起，返回0，不点亮行走路径灯
      }
    }
  }
  return 0;
}

void TowardsLED(uint8_t x0, uint8_t y0, uint8_t x, uint8_t y) {    //行走路径光
  uint8_t i, j;
  uint32_t color;
  if(attack == 2){
    color = RED;
  }else{
    color = BLUE;
  }
  attack = 0;
  if(y == y0){                 //横
    if(x > x0){                //左到右亮
      for(i = x0; i <= x; i++){
        LEDSwitch(i, y, color, DELAYVAL);    //点亮
      }
      for(i = x0; i <= x; i++){
        LEDSwitch(i, y, OFF, DELAYVAL);   //关闭
      }
    }else{                     //右到左亮
      for(i = x0; i >= x; i--){
        LEDSwitch(i, y, color, DELAYVAL);    //点亮
      }
      for(i = x0; i >= x; i--){
        LEDSwitch(i, y, OFF, DELAYVAL);    //关闭
      }
    }
  }else if(x == x0){            //直
    if((y > y0)){               //下到上亮
      for(i = y0; i <= y; i++){
        LEDSwitch(x, i, color, DELAYVAL);     //点亮
      }
      for(i = y0; i <= y; i++){
        LEDSwitch(x, i, OFF, DELAYVAL);    //关闭
      }
    }else{                      //上到下亮
      for(i = y0; i >= y; i--){
        LEDSwitch(x, i, color, DELAYVAL);     //点亮
      }
      for(i = y0; i >= y; i--){
        LEDSwitch(x, i, OFF, DELAYVAL);    //关闭
      }
    }
  }else if(((x - x0) - (y - y0) == 0)||((x - x0) + (y - y0) == 0)){    //斜
    if((x > x0) && (y > y0)){            //右上斜
      for(i = x0, j = y0; i <= x; i++, j++){
        LEDSwitch(i, j, color, DELAYVAL);              //点亮
      }
      for(i = x0, j = y0; i <= x; i++, j++){
        LEDSwitch(i, j, OFF, DELAYVAL);             //关闭
      }
    }else if((x < x0) && (y < y0)){      //左下斜
      for(i = x0, j = y0; i >= x; i--, j--){
        LEDSwitch(i, j, color, DELAYVAL);              //点亮
      }
      for(i = x0, j = y0; i >= x; i--, j--){
        LEDSwitch(i, j, OFF, DELAYVAL);              //关闭
      }
    }else if((x < x0) && (y > y0)){      //左上斜
      for(i = x0, j = y0; i >= x; i--, j++){
        LEDSwitch(i, j, color, DELAYVAL);              //点亮
      }
      for(i = x0, j = y0; i >= x; i--, j++){
        LEDSwitch(i, j, OFF, DELAYVAL);              //关闭
      }
    }else{                               //右下斜
      for(i = x0, j = y0; i <= x; i++, j--){
        LEDSwitch(i, j, color, DELAYVAL);              //点亮
      }
      for(i = x0, j = y0; i <= x; i++, j--){
        LEDSwitch(i, j, OFF, DELAYVAL);              //关闭
      }
    }
  }else{     //直斜
    LEDSwitch(x0, y0, color, DELAYVAL);      //点亮
    if((y - y0) == 2){
      LEDSwitch(x0, y0+1, color, DELAYVAL);  //点亮(x0, y0+1)
    }else if((y - y0) == -2){
      LEDSwitch(x0, y0-1, color, DELAYVAL);  //点亮(x0, y0-1)
    }else if((x - x0) == 2){
      LEDSwitch(x0+1, y0, color, DELAYVAL);  //点亮(x0+1, y0)
    }else{
      LEDSwitch(x0-1, y0, color, DELAYVAL);  //点亮(x0-1, y0)
    }
    LEDSwitch(x, y, color, DELAYVAL );
    LEDSwitch(x0, y0, OFF, DELAYVAL);
    if((y - y0) == 2){          //关闭
      LEDSwitch(x0, y0+1, OFF, DELAYVAL);  //关闭(x0, y0+1)
    }else if((y - y0) == -2){
      LEDSwitch(x0, y0-1, OFF, DELAYVAL);  //关闭(x0, y0-1)
    }else if((x - x0) == 2){
      LEDSwitch(x0+1, y0, OFF, DELAYVAL);  //关闭(x0+1, y0)
    }else{
      LEDSwitch(x0-1, y0, OFF, DELAYVAL);  //关闭(x0-1, y0)
    }
    LEDSwitch(x, y, OFF, DELAYVAL);
  }
}

void LEDSwitch(uint8_t x, uint8_t y, uint32_t color, uint8_t Dtime){  //操作灯光
  LED.setPixelColor(8*x+y-9, color);
  LED.show();
  delay(Dtime);
}

void setup() {
  //初始化灯
  LEDInit();
  //初始化扩展板
  IOInit();
  //等待棋子摆放完成, 如果没有摆放完成将在里面循环
  PiecesInit();
  //初始化坐标
  CoordinatesInit();
}

void loop() {
  uint8_t flag = 0;                 //行动标志位
  IOTraverse();                     //扫描所有IO口
  flag = Coordinatecalculation();   //拿放.位置计算,放下棋子返回1，才点亮灯
  if(flag){
    delay(500);
    TowardsLED(x0, y0, x, y);  //行走路径光
  }
}
